/* uDig - User Friendly Desktop Internet GIS client
 * http://udig.refractions.net
 * (C) 2004, Refractions Research Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */
package net.refractions.udig.tools.edit.commands;

import java.util.Collections;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.command.AbstractCommand;
import net.refractions.udig.project.command.UndoableMapCommand;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.commands.edit.AddFeatureCommand;
import net.refractions.udig.tool.edit.internal.Messages;
import net.refractions.udig.tools.edit.support.EditGeom;
import net.refractions.udig.tools.edit.support.EditUtils;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.geotools.feature.Feature;
import org.geotools.filter.Filter;
import org.geotools.filter.FilterFactoryFinder;

/**
 * Creates a new feature and sets it as the EditFeature
 * @author jones
 * @since 1.1.0
 */
public class CreateAndSetNewFeature extends AbstractCommand implements UndoableMapCommand {

    private Layer layer;
    private Feature feature;
    private AddFeatureCommand addFeatureCommand;
    private Feature oldFeature;
    private ILayer oldLayer;
    private EditGeom geom;
    private String oldID;
    private Filter oldSelection;

    public CreateAndSetNewFeature( EditGeom geom, Feature feature, ILayer layer ) {
        this.layer=(Layer) layer;
        this.feature=feature;
        this.geom=geom;
    }

    public void run( IProgressMonitor monitor ) throws Exception {
        monitor.beginTask(getName(), 14);
        monitor.worked(2);
        boolean prev = layer.eDeliver();
        try{
            layer.eSetDeliver(false);
            addFeatureCommand=new AddFeatureCommand(feature, layer);
            addFeatureCommand.setMap(getMap());
            SubProgressMonitor subProgressMonitor = new SubProgressMonitor(monitor, 10);
            addFeatureCommand.run(subProgressMonitor);
            subProgressMonitor.done();
            this.oldFeature=getMap().getEditManager().getEditFeature();
            this.oldLayer=getMap().getEditManager().getEditLayer();
            this.oldID=geom.getFeatureIDRef().get();
            geom.getFeatureIDRef().set(addFeatureCommand.getFid());
    
            getMap().getEditManagerInternal().setEditFeature(addFeatureCommand.getNewFeature(), layer);
            oldSelection = layer.getFilter();
            Filter filter = FilterFactoryFinder.createFilterFactory().createFidFilter(addFeatureCommand.getFid());
            layer.setFilter(filter);
            
            EditUtils.instance.refreshLayer(layer, Collections.singleton(addFeatureCommand.getFid()), null, false, true);
        }finally{
            layer.eSetDeliver(prev);
        }

        monitor.done();
    }

    public String getName() {
        return Messages.CreateAndSetNewFeature_name;
    }

    public void rollback( IProgressMonitor monitor ) throws Exception {
        monitor.beginTask(getName(), 14);
        monitor.worked(2);
        boolean prev = layer.eDeliver();
        try{
            layer.eSetDeliver(false);
            layer.setFilter(oldSelection);
            
            SubProgressMonitor subProgressMonitor = new SubProgressMonitor(monitor, 10);
            addFeatureCommand.rollback(subProgressMonitor);
            subProgressMonitor.done();
            geom.getFeatureIDRef().set(oldID);
    
            getMap().getEditManagerInternal().setEditFeature(oldFeature, (Layer)oldLayer);
            EditUtils.instance.refreshLayer(layer, Collections.singleton(addFeatureCommand.getFid()), null, false, false);
        }finally{
            layer.eSetDeliver(prev);
        }
        monitor.done();
    }

}
